#module  LIB_GET_DISK_QUOTA  "V1.3"

/*|++GETDSKQUO.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET
	General procedure library.

 Abstract:
	Return the disk quota entry including the current disk usage for
	the user (or a specified UIC).

 Environment:
	Called from user mode.
|--*/
/*
 Modification History:
 Author: F. Nagy			Creation date: 07-Jul-83

 V1.0	07-Jul-83  FJN	Created for use by Data Logger
 V1.1	23-Jul-83  FJN	Fix setup of usrchan boolean by testing chan
			argument for NULL, not disk!
 V1.2	26-Jul-83  FJN	Cannot call more than once without error being returned,
			fixed by zeroing FIB$L_WCC!!!!!!
 V1.3	14-Aug-84  FJN	Eliminated LIB_NARGS call by using pointer trick
*/

/* Include Files: */
#include vaxtypes			/* General type definitions */
#include descrip			/* Descriptors */
#include descrip2			/* More on descriptors */
#include iodef				/* I/O functions codes/modifiers */
#include jpidef				/* $GETJPI item codes */
#include $dqfdef			/* Disk quota record */
#include $fibdef			/* File Information Block */

/* Local definitions and types: */
typedef unsigned long *ulref;		/* Unsigned longword passed by ref. */
typedef struct dsc$descriptor_s *descr;	/* Pointer to descriptor, or string
					   passed by descriptor */

/* Module Local Own Storage: */
globalvalue SS$_NORMAL;


/*+ _GET_DISK_QUOTA
 Get disk quota and usage information.

 status.wlc.v = LIB_GET_DISK_QUOTA( usage.wlu.r, perm.wlu.r
				    [,disk.rt.dx1] [,uic.mlu.r]
				    [,chan.mwu.r] [,over.wlu.r] )

  status  returns completion status.  Passed by value.

  usage   returns the current disk usage (in blocks).  Passed by
          reference.

  perm    returns the permanent disk quota (in blocks).  Passed by
          reference.

  disk    optionally specifies the name of the disk (or a logical
          name) on which the quota entry to be returned resides.
          If defaulted, "SYS$DISK:" is used.  Passed by scalar
          descriptor.

  uic     optionally specifies the UIC for the quota entry.  If
          defaulted, the current UIC of the process is used.  If
          a longword value of 0 is specified, the current UIC is
          again used and then returned in the longword (replacing
          the 0) for use in later calls.

  chan    optionally specifies a word in which the channel number
          assigned to the disk device.  If defaulted, a channel is
          assigned and then deassigned for each call.  If specified
          with a value of 0, a channel is assigned and returned in
          the argument for use with later calls.

  over    optionally returns the overdraft disk quota (in blocks).
          Passed by reference.

2 Examples
 These are some examples of the use of this routine.

 To get the disk usage and permanent quota of the current UIC on the
 current default disk (sort of once only):

        status = LIB_GET_DISK_QUOTA( dusage,dperm )

 To get the disk usage and permanent quota of the current UIC on some
 specified device (given by a logical name):

        status = LIB_GET_DISK_QUOTA( dusage,dperm,'OTHER_DISK:' )

 To repeated call to monitor disk usage and remaining quota on a disk
 for the current UIC, where in the initialization code the variables
 thisuic and qchannel were initialized to 0:

        status = LIB_GET_DISK_QUOTA( dusage,dperm,'APPLE_DISK:',
                                     thisuic,qchannel )

 On the first call, thisuic and qchannel will be written with the
 current UIC and the channel on which APPLE_DISK has been opened.
 These variables are then saved (non-stack variables) to eliminate
 the overhead to lookup UIC and open disk channel on subsequent
 calls.

2 Status
 Some of the completion status codes returned are:

    SS$_NORMAL      success
    SS$_ACCVIO      device string or descriptor cannot be read
    SS$_BADQFILE    invalid disk quota file format
    SS$_DEVALLOC    device is allocated to another process
    SS$_DEVOFFLINE  device is offline
    SS$_IVCHAN      invalid channel number
    SS$_IVDEVNAM    invalid or non-existant device name
    SS$_IVLOGNAM    invalid logical name
    SS$_NODISKQUOTA no disk quota entry for UIC
    SS$_NOIOCHAN    no I/O channel is available for assignment
    SS$_NOPRIV      non-existant channel or no privilege to
                    read quota file
    SS$_NOSUCHDEV   (warning) no such device
    SS$_QFNOTACT    disk quota file is not active

-*/
/*+0LIB_GET_DISK_QUOTA

 Functional Description:
	Open a channel to a disk device and use the ACPCONTROL I/O function
	to return a specified (by UIC) disk quota file entry.

 Input Parameters:
	diskname - optionally specifies the device name of the disk on which
		   the quota file resides.  If defaulted, the current default
		   device (SYS$DISK:) is used.
	uic - optionally specifies the UIC of the disk quota record to be
	      accessed.  If defaulted, then the current process UIC is used.
	      If specified as 0, the current process UIC is used.
	chan - optionally specifies the channel number which the disk
	       device has been assigned by the user or by a previous call.
	       If defaulted, a channel is assigned and deassigned on each call.

 Implicit Inputs:
	NONE

 Output Parameters:
	usage - returns the current disk usage on the specified or default
		device.
	perm - returns the permanent disk quota on the same device.
	uic - if specified as 0 on input, then the current process UIC is
	      returned in this argument.
	chan - if specified as 0 on input, then a channel is assigned to the
	       specified disk and the channel number is returned in this
	       argument.
	over - optionally returns the overdraft quota on the same device.

 Implicit Outputs:
	NONE

 Condition Codes:
	SS$_NORMAL	success
	SS$_ACCVIO	device string or descriptor cannot be read
	SS$_BADQFILE	invalid disk quota file format
	SS$_DEVALLOC	device is allocated to another process
	SS$_DEVOFFLINE	device is offline
	SS$_IVCHAN	invalid channel number
	SS$_IVDEVNAM	invalid or non-existant device name
	SS$_IVLOGNAM	invalid logical name
	SS$_NODISKQUOTA	no disk quota entry for UIC
	SS$_NOIOCHAN	no I/O channel is available for assignment
	SS$_NOPRIV	non-existant channel or no privilege to read quota file
	SS$_NOSUCHDEV	(warning) no such device
	SS$_QFNOTACT	disk quota file is not active

 Side Effects:
	Event flag 0 is used.
-*/

#define  NARG_DISK  3
#define  NARG_UIC   4
#define  NARG_CHAN  5
#define  NARG_OVER  6

ulong LIB_GET_DISK_QUOTA( usage, perm, diskname, uic, chan, over )
ulref usage,perm,over;			/* Returns usage and quotas */
ulref uic;				/* Optionally specified/returned UIC */
uword *chan;				/* Optionally specified/returned
					   channel number for disk. */
descr diskname;				/* Optionally names a disk device */
{
ulong SYS$QIOW();			/* Queue I/O operation service */
ulong SYS$GETJPI();			/* Get job/process info. service */
ulong SYS$ASSIGN();			/* Assign I/O channel service */
ulong SYS$DASSGN();			/* Release I/O channel service */
register ubyte nargs;			/* Holder for argument count */
register ulong status;			/* Completion status */
uword iosb[4];				/* I/O status block */
struct DQF qrec;			/* Quota record */
struct CONTROL_FIB fib;			/* File Information Block for ACP
					   control operation */
					/* Default device */
static readonly $DESCRIPTOR( _default, "SYS$DISK:");
uword channel;				/* Holds channel number */
register descr devptr;			/* Pointer to device descriptor */
$DESCRIPTOR1( _fib, fib);		/* Fib descriptor */
$DESCRIPTOR1( _qrec, qrec);		/* Quota record descriptor */
struct {				/* Item list for $GETJPI */
    uword jpi_buflen;			    /* Buffer length */
    uword jpi_itmcod;			    /* Item code */
    ulong *jpi_bufadr;			    /* Pointer to longword buffer */
    ulong *jpi_rtnlen;			    /* Pointer to return length */
    ulong jpi_end;			    /* Terminator code */
    } itemlist = { 4, JPI$_UIC, &qrec.dqf$l_uic, 0, JPI$C_LISTEND };
register byte usrchan;			/* Flag is TRUE if user specified
					   channel word argument */

/*
  Initialize fields of the FIB and Quota records (DQF).  Also get the number
  of arguments in the call.
*/
fib.fib$w_cntrlfunc = FIB$C_EXA_QUOTA;	/* Examine quota function */
fib.fib$l_cntrlval = 0;
fib.fib$l_wcc = 0;			/* Clear wildcard context */
qrec.dqf$l_flags = 0;

nargs = *( (long *) &usage - 1);	/* Get number of arguments */

/*
  Set the UIC to either the UIC specified in the argument list or to the
  UIC of the current process (if none explicitly specified).  If argument
  specified as 0, get and return current process UIC.
*/
if ( (nargs >= NARG_UIC) && (uic != NULL) && (*uic != 0) )
    qrec.dqf$l_uic = *uic;
else
    {
    status = SYS$GETJPI( 0, 0, 0, &itemlist, 0, 0, 0);
    if ($VMS_FAILURE( status))
	return status;
    if ( (nargs >= NARG_UIC) && (uic != NULL))
	*uic = qrec.dqf$l_uic;
    };

/*
  Open channel to disk with quota entries.  If no explicit device specified
  in the arguments, then SYS$DISK: is used.  If user provided a channel number,
  then use that rather than opening a new channel.  If the user's channel word
  contains 0, open the channel and return the channel number.
*/
usrchan = (nargs >= NARG_CHAN) && (chan != NULL);
if ( usrchan && (*chan != 0))
    channel = *chan;
else
    {
    if ( (nargs >= NARG_DISK) && (diskname != NULL) )
	devptr = diskname;
    else
	devptr = &_default;

    status = SYS$ASSIGN( devptr, &channel, 0, 0);
    if ($VMS_FAILURE( status))
	return status;

    if (usrchan)
	*chan = channel;
    };

/*
  Now perform ACP control function to examine quota file for specific entry
  on the now-opened disk channel.  Then close and release the channel.
*/
status = SYS$QIOW( 0, channel, IO$_ACPCONTROL, iosb, 0, 0,
		   &_fib, &_qrec, 0, &_qrec, 0, 0);
if ($VMS_FAILURE( status))
	return status;
if ($VMS_FAILURE( iosb[0]))
	return iosb[0];

if (!usrchan)
    {
    status = SYS$DASSGN( channel);
    if ($VMS_FAILURE( status))
	return status;
    };

/*
  Successfully got the quota entry, now return the various values.
*/
*usage = qrec.dqf$l_usage;
*perm = qrec.dqf$l_permquota;
if ( (nargs >= NARG_OVER) && (over != NULL) )
    *over = qrec.dqf$l_overdraft;

return SS$_NORMAL;
}
