#module  LIB_HIBERSX  "V1.0"

/*|++LIBHIBERS.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET

 Abstract:
	Routines to wait (sleep) for a specified time using $HIBER.

 Environment:
	General library of additional runtime routines.
|--*/
/*
 Modification History:
 Author: F. Nagy

 V1.0	01-Aug-85  FJN	Created to complement LIBSLEEP.C using the old method
			of $SCHDWK/$HIBER to allow the delay to be shorted by
			the user from an AST
*/

/******************
 * Include Files: *
 ******************/
#include  vaxtypes			/* VAX-11 C Type extensions */

/********************************
 * Local definitions and types: *
 ********************************/
typedef unsigned long status;		/* Condition code status type */

					/* System time units... */
#define	TICKS_PER_SECOND	(-10*1000*1000)
#define	TICKS_PER_MILLISECOND	(-10*1000)

/**********************************************
 * Global Procedure and Function definitions: *
 **********************************************/
status SYS$SCHDWK();			/* Schedule wakeup system service */
status SYS$HIBER();			/* Hibernate system service */
status LIB$EMUL();			/* Extended Multiply instruction */


/*+ LIB_HIBERS
 Put process to sleep until an absolute time or delay process
 execution for a specified amount of time.

  status.wlc.v = LIB_HIBERS( time.rq.r )

 status    completion status (returned in R0) from the $SCHDWK
           system service.

 time      quadword containing system-format time or delta-time
           which specifies the delay (or sleep-until) time.

 Note: the delay is implemented using $SCHDWK to schedule a wakeup and
 $HIBER to wait for the wakeup.  This delay time can be cut short by
 calling $WAKE in an AST routine (in such a case there the scheduled
 wakeup will still cause a wake-pending to be set when the delay expires
 thus negating the next call to $HIBER; this can be eliminated by calling
 $CANWAK to cancel the scheduled wakeup).

 Due to other system activity and the clock granularity, the length of
 time that the process actually waits may be a bit longer than what
 was specified by the call argument.
-*/

status LIB_HIBERS( delay)
vmstime delay;				/* Delay time in system format */
{
register status sts;			/* Temporary for status */

/*
 Set scheduled wakeup for the future after the delay time.
*/
sts = SYS$SCHDWK( 0, 0, delay, 0);
RETURN_ON_FAILURE( sts);

/*
 Wait for timer to expire - process now goes to sleep...
*/
return SYS$HIBER();
}


/*+ _HIBERS_SECONDS
 Put process to sleep for a specified number of seconds.

  status.wlc.v = LIB_HIBERS_SECONDS( seconds.rlu.r )

 status    completion status (returned in R0) from the $SCHDWK
           system service.

 seconds   unsigned quantity giving the number of seconds to delay
           before continuing execution.

 Note: the delay is implemented using $SCHDWK to schedule a wakeup and
 $HIBER to wait for the wakeup.  This delay time can be cut short by
 calling $WAKE in an AST routine (in such a case there the scheduled
 wakeup will still cause a wake-pending to be set when the delay expires
 thus negating the next call to $HIBER; this can be eliminated by calling
 $CANWAK to cancel the scheduled wakeup).

 Due to other system activity and the clock granularity, the length of
 time that the process actually waits may be a bit longer than what
 was specified by the call argument.
-*/

status LIB_HIBERS_SECONDS( seconds)
int *seconds;				/* Number of seconds to sleep */
{
register status sts;			/* Temporary for status */
vmstime delay;				/* To get delay in system units */

/*
 Convert sleep time in seconds to sleep time delay in system units.
*/
sts = LIB$EMUL( seconds, &TICKS_PER_SECOND, &0, delay);
RETURN_ON_FAILURE( sts);

/*
 Sleep until delay expires...
*/
return LIB_HIBERS( delay);
}


/*+ _HIBERS_MILLISE
 Put process to sleep for a specified number of milliseconds.

  status.wlc.v = LIB_HIBERS_MILLISECS( msecs.rlu.r )

 status    completion status (returned in R0) from the $SCHDWK
           system service.

 msecs     unsigned quantity giving the number of milliseconds to
           delay before continuing execution.

 Note: the delay is implemented using $SCHDWK to schedule a wakeup and
 $HIBER to wait for the wakeup.  This delay time can be cut short by
 calling $WAKE in an AST routine (in such a case there the scheduled
 wakeup will still cause a wake-pending to be set when the delay expires
 thus negating the next call to $HIBER; this can be eliminated by calling
 $CANWAK to cancel the scheduled wakeup).

 Due to other system activity and the clock granularity, the length of
 time that the process actually waits may be a bit longer than what
 was specified by the call argument.
-*/

status LIB_HIBERS_MILLISECS( msecs)
int *msecs;				/* Number of milliseconds to sleep */
{
register status sts;			/* Temporary for status */
vmstime delay;				/* To get delay in system units */

/*
 Convert sleep time in milliseconds to sleep time delay in system units.
*/
sts = LIB$EMUL( msecs, &TICKS_PER_MILLISECOND, &0, delay);
RETURN_ON_FAILURE( sts);

/*
 Sleep until delay expires...
*/
return LIB_HIBERS( delay);
}
